@@ -1,13 +1,14 @@
 import datetime
 import functools
 import json
+from decimal import ROUND_HALF_UP, Decimal
 from enum import Enum
 from json import JSONEncoder
 from typing import List, Optional
 from uuid import UUID
 
 from dateutil import parser
-from pydantic import BaseModel, ValidationError, constr, validator
+from pydantic import BaseModel, ValidationError, condecimal, constr, root_validator, validator
 
 from ..utils.logger import payment_logger as logger
 
@@ -17,10 +18,10 @@ def check_iso_format(v):
         try:
             return parser.parse(v)  # Converte stringa in datetime
         except Exception:
-            raise ValidationError("date must be a valid ISO8601 datetime")
+            raise ValidationError('date must be a valid ISO8601 datetime')
     if isinstance(v, datetime.datetime):
         return v  # Se è già un datetime, lo lascia invariato
-    raise ValidationError("expire_at must be a datetime or an ISO8601 string")
+    raise ValidationError('expire_at must be a datetime or an ISO8601 string')
 
 
 def check_isoformat_none_case(v):
@@ -29,10 +30,10 @@ def check_isoformat_none_case(v):
             try:
                 return parser.parse(v)  # Converte stringa in datetime
             except Exception:
-                raise ValidationError("date must be a valid ISO8601 datetime")
+                raise ValidationError('date must be a valid ISO8601 datetime')
         if isinstance(v, datetime.datetime):
             return v  # Se è già un datetime, lo lascia invariato
-        raise ValidationError("date must be a datetime or an ISO8601 string")
+        raise ValidationError('date must be a datetime or an ISO8601 string')
 
 
 class HttpMethodType(Enum):
@@ -87,12 +88,18 @@ class PayerType(Enum):
 
 class PaymentDataSplit(BaseModel):
     code: str
-    amount: Optional[float]
+    amount: condecimal(max_digits=12, decimal_places=2)
     meta: dict
 
-    class Config:
-        validate_all = True
-        validate_assignment = True
+    @validator('amount', pre=True)
+    def _parse_split_amount(cls, v):
+        if v is None:
+            return v
+        return Decimal(str(v)).quantize(Decimal('0.01'), rounding=ROUND_HALF_UP)
+
+    @property
+    def amount_cents(self) -> int:
+        return int((self.amount * 100).quantize(Decimal('1'), rounding=ROUND_HALF_UP))
 
     def json(self):
         return json.dumps(self, cls=PaymentEncoder)
@@ -134,7 +141,7 @@ class PaymentData(BaseModel):
     transaction_id: Optional[str]
     paid_at: Optional[datetime.datetime]
     expire_at: Optional[datetime.datetime]
-    amount: Optional[float]
+    amount: Optional[condecimal(max_digits=12, decimal_places=2)]
     currency: CurrencyType
     type: Optional[PaymentType]
     notice_code: Optional[str]
@@ -153,6 +160,41 @@ class PaymentData(BaseModel):
     paid_at_must_be_iso8601 = validator('paid_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
     expire_at_must_be_iso8601 = validator('expire_at', pre=True, allow_reuse=True)(check_isoformat_none_case)
 
+    @validator('amount', pre=True)
+    def _parse_amount(cls, v):
+        if v is None:
+            return v
+        return Decimal(str(v)).quantize(Decimal('0.01'), rounding=ROUND_HALF_UP)
+
+    @property
+    def amount_cents(self) -> Optional[int]:
+        if self.amount is None:
+            return None
+        return int((self.amount * 100).quantize(Decimal('1'), rounding=ROUND_HALF_UP))
+
+    @root_validator(pre=False)
+    def _check_splits_total(cls, values):
+        amt = values.get('amount')  # Decimal | None
+        splits = values.get('split') or []  # List[PaymentDataSplit] | []
+
+        # Se manca amount o non ci sono split, non validiamo la somma
+        if amt is None or not splits:
+            return values
+
+        total = sum((s.amount for s in splits if s is not None), Decimal('0'))
+
+        # Confronto commerciale a 2 decimali
+        total_q = total.quantize(Decimal('0.01'))
+        amt_q = amt.quantize(Decimal('0.01'))
+
+        # Accetta:
+        # 1) split come importi: somma == amount
+        # 2) (opzionale) split come pesi: somma == 1.00
+        if not (total_q == amt_q or total_q == Decimal('1.00')):
+            raise ValueError(f'Somma degli split {total_q} diversa da amount {amt_q}')
+
+        return values
+
     def json(self):
         return json.dumps(self, cls=PaymentEncoder)
 
@@ -396,6 +438,8 @@ class PaymentEncoder(JSONEncoder):
             return o.isoformat()
         elif isinstance(o, UUID):
             return str(o)
+        elif isinstance(o, Decimal):
+            return float(o.quantize(Decimal('0.01')))
         elif isinstance(o, (PaymentType, PayerType, PaymentStatus, HttpMethodType, CurrencyType)):
             return o.value
         return o.__dict__
