@@ -1,5 +1,6 @@
 import json
 from datetime import datetime, timedelta, timezone
+from decimal import Decimal
 from unittest import TestCase
 
 from freezegun import freeze_time
@@ -62,7 +63,7 @@ class PaymentTestCase(TestCase):
         self.assertEqual(payment.payment.transaction_id, None)
         self.assertIsNone(payment.payment.paid_at)
         self.assertEqual(payment.payment.expire_at, datetime.fromisoformat('2023-09-04T11:59:05+02:00'))
-        self.assertEqual(payment.payment.amount, 16)
+        self.assertEqual(payment.payment.amount, 16.50)
         self.assertEqual(payment.payment.currency, CurrencyType.CURRENCY_EUR)
         self.assertEqual(payment.payment.type, PaymentType.TYPE_STAMP)
         self.assertEqual(payment.payment.notice_code, '001550000000024427')
@@ -82,10 +83,10 @@ class PaymentTestCase(TestCase):
         self.assertIsNotNone(payment.payment.document)
         self.assertEqual(len(payment.payment.split), 2)
         self.assertEqual(payment.payment.split[0].code, 'c_1')
-        self.assertEqual(payment.payment.split[0].amount, 0.5)
+        self.assertEqual(payment.payment.split[0].amount, 9)
         self.assertEqual(payment.payment.split[0].meta, {})
         self.assertEqual(payment.payment.split[1].code, 'c_2')
-        self.assertEqual(payment.payment.split[1].amount, 0.5)
+        self.assertEqual(payment.payment.split[1].amount, 7.5)
         self.assertEqual(payment.payment.split[1].meta, {})
 
         # Asserzioni per l'oggetto Links
@@ -317,3 +318,131 @@ class PaymentTestCase(TestCase):
             PaymentData(**get_payment_data(split=get_split_data_invalid()))
         with self.assertRaises(ValidationError):
             PaymentData(**get_payment_data(split='Lorem ipsum'))
+
+    def test_decimal_amount_parsing_and_cents_from_float(self):
+        pd = PaymentData(
+            transaction_id=None,
+            paid_at=None,
+            expire_at='2025-10-10T19:59:02+02:00',
+            amount=32.05,  # arriva come float
+            currency=CurrencyType.CURRENCY_EUR,
+            type=PaymentType.TYPE_STAMP,
+            notice_code='303000000004610406',
+            iud='68dada7823984a11b80a98aaede3371c',
+            iuv='03000000004610406',
+            receiver=None,
+            due_type='TD_SALE',
+            pagopa_category='n/a',
+            document=None,
+            split=[
+                {'code': 'c_bollo_istanza', 'amount': 32, 'meta': {}},
+                {'code': 'c_altro_costo', 'amount': 0.05, 'meta': {}},
+            ],
+        )
+        self.assertIsInstance(pd.amount, Decimal)
+        self.assertEqual(pd.amount, Decimal('32.05'))
+        self.assertEqual(pd.amount_cents, 3205)
+
+        # split
+        self.assertIsInstance(pd.split[0].amount, Decimal)
+        self.assertEqual(pd.split[0].amount, Decimal('32.00'))
+        self.assertEqual(pd.split[0].amount_cents, 3200)
+        self.assertEqual(pd.split[1].amount, Decimal('0.05'))
+        self.assertEqual(pd.split[1].amount_cents, 5)
+
+    def test_decimal_sum_of_splits_must_match_amount_ok(self):
+        pd = PaymentData(
+            transaction_id=None,
+            paid_at=None,
+            expire_at=None,
+            amount='10.30',  # arriva come stringa
+            currency=CurrencyType.CURRENCY_EUR,
+            type=PaymentType.TYPE_STAMP,
+            notice_code='303000000004610406',
+            iud='68dada7823984a11b80a98aaede3371c',
+            iuv='03000000004610406',
+            receiver=None,
+            due_type=None,
+            pagopa_category=None,
+            document=None,
+            split=[
+                {'code': 'A', 'amount': '10.00', 'meta': {}},
+                {'code': 'B', 'amount': '0.30', 'meta': {}},
+            ],
+        )
+        self.assertEqual(pd.amount, Decimal('10.30'))
+        self.assertEqual(sum(s.amount for s in pd.split if s), Decimal('10.30'))
+
+    def test_decimal_sum_of_splits_must_match_amount_raises(self):
+        with self.assertRaises(ValidationError):
+            PaymentData(
+                transaction_id=None,
+                paid_at=None,
+                expire_at=None,
+                amount=32.05,
+                currency=CurrencyType.CURRENCY_EUR,
+                type=PaymentType.TYPE_STAMP,
+                notice_code='303000000004610406',
+                iud='68dada7823984a11b80a98aaede3371c',
+                iuv='03000000004610406',
+                receiver=None,
+                due_type=None,
+                pagopa_category=None,
+                document=None,
+                # 32.00 + 0.04 = 32.04 != 32.05
+                split=[
+                    {'code': 'A', 'amount': 32, 'meta': {}},
+                    {'code': 'B', 'amount': 0.04, 'meta': {}},
+                ],
+            )
+
+    def test_decimal_rounding_half_up_edge_case_1005(self):
+        s = PaymentDataSplit(code='edge', amount='1.005', meta={})
+        # con ROUND_HALF_UP → 1.01 e 101 cent
+        self.assertEqual(s.amount, Decimal('1.01'))
+        self.assertEqual(s.amount_cents, 101)
+
+    def test_decimal_eliminates_float_drift_point1_plus_point2(self):
+        # 0.1 + 0.2 = 0.3 (nessuna deriva binaria con Decimal)
+        pd = PaymentData(
+            transaction_id=None,
+            paid_at=None,
+            expire_at=None,
+            amount=0.3,
+            currency=CurrencyType.CURRENCY_EUR,
+            type=PaymentType.TYPE_STAMP,
+            notice_code=None,
+            iud='68dada7823984a11b80a98aaede3371c',
+            iuv=None,
+            receiver=None,
+            due_type=None,
+            pagopa_category=None,
+            document=None,
+            split=[
+                {'code': 'A', 'amount': 0.1, 'meta': {}},
+                {'code': 'B', 'amount': 0.2, 'meta': {}},
+            ],
+        )
+        self.assertEqual(pd.amount, Decimal('0.30'))
+        self.assertEqual(pd.amount_cents, 30)
+        self.assertEqual(sum(s.amount for s in pd.split if s), Decimal('0.30'))
+
+    def test_amount_cents_none_when_amount_none(self):
+        pd = PaymentData(
+            transaction_id=None,
+            paid_at=None,
+            expire_at=None,
+            amount=None,
+            currency=CurrencyType.CURRENCY_EUR,
+            type=None,
+            notice_code=None,
+            iud='iud-decimal-5',
+            iuv=None,
+            receiver=None,
+            due_type=None,
+            pagopa_category=None,
+            document=None,
+            split=[],  # vuota: la root validation non scatta
+        )
+        self.assertIsNone(pd.amount)
+        self.assertIsNone(pd.amount_cents)
